/*	AMX Mod X script.

	IP converter by Zetex.
	Developed for Subnet Ban project.
	Special thanks to Lev and Zefir.

	is_valid_ip(ip_string[])
		Checks if given IP is valid.
		Return true or false.

	is_valid_subnet(net_string[])
		Checks if given subnet is valid.
		Return true or false.

	ip_to_long(ip_string[])
		Convert IP to LONG.
		Return unsigned long.

	long_to_ip(ip_num)
		Convert LONG to IP.
		Return string[16]

	cidr_to_long(mask_string[])
		Convert mask to LONG.
		Return unsigned long.

	long_to_cidr(mask_num)
		Convert LONG to mask.
		Return string[3]

	range1(net_string[], &net, &mask)
		Get net and mask as LONG from subnet.

	range2(net_string[], &startip, &endip)
		Get startip and endip as LONG from subnet.
*/

#if defined _ip_stocks_included
  #endinput
#endif
#define _ip_stocks_included

stock bool:is_valid_ip(ip_string[])
{
	new right[16], part[4], octet, count = 0;
	strtok(ip_string, part, 3, right, 15, '.');

	while (part[0] != 0)
	{
		octet = str_to_num(part);

		if (octet < 0 || octet > 255)
		{
			return false;
		}

		count++;
		strtok(right, part, 3, right, 15, '.');
	}

	return count <= 4;
}

stock bool:is_valid_subnet(net_string[])
{
	new i, ip[16], mask;

	if ((i = contain(net_string, "/")) != -1)
	{
		copy(ip, i - 1, net_string);
		mask = str_to_num(net_string[i + 1]);
	}

	if (!is_valid_ip(ip) || mask < 0 || mask > 32)
	{
		return false;
	}

	return true;
}

stock ip_to_long(ip_string[])
{	
	new right[16], part[4], octet, ip = 0;
	strtok(ip_string, part, 3, right, 15, '.');

	for (new i = 0; i < 4; i++)
	{
		octet = str_to_num(part);

		if (octet < 0 || octet > 255)
		{
			octet = 255;
		}

		ip += octet;

		if (i == 3)
		{
			break;
		}

		strtok(right, part, 3, right, 15, '.');
		ip = ip << 8;
	}

	return ip;
}

stock long_to_ip(ip_num)
{
	new octet[4], bool:high, ip[16];

	if (ip_num < 0)
	{
		high = true;
		ip_num = ip_num & (~(1 << 31));
	}

	for (new i = 0; i < 4; i++)
	{
		octet[i] = ip_num & 255;
		ip_num = ip_num >> 8;
	}

	if (high)
	{
		octet[3] += 128;
	}

	formatex(ip, 15, "%i.%i.%i.%i", octet[3], octet[2], octet[1], octet[0]);
	return ip;
}

stock cidr_to_long(mask_string[])
{
	new mask = str_to_num(mask_string);
	new result = (1 << 31) >> mask;

	return result;
}

stock long_to_cidr(mask_num)
{
	new i = 0, result[3];

	while (!(mask_num & 1) && i++ < 31)
	{
		mask_num >>>= 1;
	}

	num_to_str(31 - i, result, 2);
	return result;
}

stock range1(net_string[], &net, &mask)
{
	new i, ip[16];

	if ((i = contain(net_string, "/")) != -1)
	{
		copy(ip, i - 1, net_string);
		mask = cidr_to_long(net_string[i + 1]);
	}

	net = ip_to_long(ip) & mask;
}

stock range2(net_string[], &startip, &endip)
{
	new i, ip[16], mask;

	if ((i = contain(net_string, "/")) != -1)
	{
		copy(ip, i - 1, net_string);
		mask = str_to_num(net_string[i + 1]);
	}

	mask = (1 << 31) >> (mask - 1);
	startip = ip_to_long(ip) & mask;
	endip = ip_to_long(ip) | (~mask);
}